@@ -2,14 +2,15 @@
 lr.b.com <- function( b, arguments )
 {
     
-    l1     <- arguments$l1
-    l2     <- arguments$l2
-    z      <- arguments$z
-    n      <- arguments$n
-    method <- arguments$method
-    crit   <- arguments$crit 
-    lambda <- arguments$lambda
-    res.df <- arguments$res.df 
+    l1       <- arguments$l1
+    l2       <- arguments$l2
+    z        <- arguments$z
+    n        <- arguments$n
+    method   <- arguments$method
+    crit     <- arguments$crit 
+    lambda   <- arguments$lambda
+    res.df   <- arguments$res.df 
+    r.factor <- arguments$r.factor
 
     if ( (method==1) | (method=='SMA') )
     {
@@ -27,6 +28,7 @@ lr.b.com <- function( b, arguments )
         l2b <- ( b^2*z[,2] - 2*b*z[,3] + z[,1] )/( lambda + b^2 )
     }
 
-    lr <- sum( (res.df - 0.5)*log( l1b*l2b/l1/l2 ), na.rm=TRUE ) - crit
+     
+    lr <- sum( ( res.df - 0.5 ) * log (1 + ( l1b*l2b/l1/l2 - 1 ) / r.factor ), na.rm=TRUE ) - crit
 }
 
